def.age		<- 12:44
def.young.age	<- 12:18
def.old.age	<- 19:44

normal_loess <- function(name, mean.span, sd.span, age=def.age) {
    data <- read.table(paste("../data/combined/", name, ".txt", sep=""))

    mean.model <- loess(V2 ~ V1, data, weights=1/V3, span=mean.span, surface="direct", statistics="exact", trace.hat="exact")
    mean.values <- predict(mean.model, age)

    sd.model <- loess(V3 ~ V1, data, weights=1/V3, span=sd.span, surface="direct", statistics="exact", trace.hat="exact")
    sd.values <- predict(sd.model, age)

    write.table(list(age=age, mean=mean.values, sd=sd.values), paste("../tables/", name, ".txt", sep=""), sep="\t", na="", row.names=FALSE, col.names=FALSE)
}

split_loess <- function(name, young.mean.span, young.sd.span,
			      old.mean.span, old.sd.span,
			      young.age=def.young.age, old.age=def.old.age) {
    age <- c(young.age, old.age)
    data <- read.table(paste("../data/combined/", name, ".txt", sep=""))

    young.mean.model <- loess(V2 ~ V1, data, weights=1/V3, span=young.mean.span, surface="direct", statistics="exact", trace.hat="exact")
    young.mean.values <- predict(young.mean.model, young.age)

    old.mean.model  <- loess(V2 ~ V1, data, weights=1/V3, span=old.mean.span, surface="direct", statistics="exact", trace.hat="exact")
    old.mean.values <- predict(old.mean.model, old.age)

    young.sd.model <- loess(V3 ~ V1, data, weights=1/V3, span=young.sd.span, surface="direct", statistics="exact", trace.hat="exact")
    young.sd.values <- predict(young.sd.model, young.age)

    old.sd.model <- loess(V3 ~ V1, data, weights=1/V3, span=old.sd.span, surface="direct", statistics="exact", trace.hat="exact")
    old.sd.values <- predict(old.sd.model, old.age)

    write.table(list(age=age, mean=c(young.mean.values, old.mean.values), sd=c(young.sd.values, old.sd.values)), paste("../tables/", name, ".txt", sep=""), sep="\t", na="", row.names=FALSE, col.names=FALSE)
}

linear <- function(name, author, age=def.age) {
    data <- read.table(paste("../data/", author, "/", name, ".txt", sep=""))

    model = nls(V2 ~ a * V1 + b, data, list(a=1, b=1))
    a = as.vector(model$m$getPars()['a'])
    b = as.vector(model$m$getPars()['b'])

    cat("u_coef = ", a, " ", b, "\n")
    cat("u_range = ", min(age), " ", max(age), "\n");

    mean.values = age * a + b
    
    write.table(list(age=age, mean=mean.values), paste("../tables/", name, ".txt", sep=""), sep="\t", na="", row.names=FALSE, col.names=FALSE)
}


quadratic_linear <- function(name, author, age=def.age) {
    data <- read.table(paste("../data/", author, "/", name, ".txt", sep=""))

    model = nls(V2 ~ a * V1 ^ 2 + b * V1 + c, data, list(a=1, b=1, c=1))
    a = as.vector(model$m$getPars()['a'])
    b = as.vector(model$m$getPars()['b'])
    c = as.vector(model$m$getPars()['c'])

    cat("u_coef   = ", a, " ", b, " ", c, "\n")
    cat("u_range  = ", min(age), " ", max(age), "\n");

    mean.values = a * age ^2 + b * age + c

    model = nls(V3 ~ a * V1 + b, data, list(a=1, b=1))
    a = as.vector(model$m$getPars()['a'])
    b = as.vector(model$m$getPars()['b'])

    cat("sd_coef  = ", a, " ", b, "\n")
    cat("sd_range = ", min(age), " ", max(age), "\n");

    sd.values = a * age + b
    
    write.table(list(age=age, mean=mean.values, sd=sd.values), paste("../tables/", name, ".txt", sep=""), sep="\t", na="", row.names=FALSE, col.names=FALSE)

}


linear_linear <- function(name, author, age=def.age) {
    data <- read.table(paste("../data/", author, "/", name, ".txt", sep=""))

    model = nls(V2 ~ a * V1 + b, data, list(a=1, b=1))
    a = as.vector(model$m$getPars()['a'])
    b = as.vector(model$m$getPars()['b'])

    cat("u_coef   = ", a, " ", b, "\n")
    cat("u_range  = ", min(age), " ", max(age), "\n");

    mean.values = a * age + b

    model = nls(V3 ~ a * V1 + b, data, list(a=1, b=1))
    a = as.vector(model$m$getPars()['a'])
    b = as.vector(model$m$getPars()['b'])

    cat("sd_coef  = ", a, " ", b, "\n")
    cat("sd_range = ", min(age), " ", max(age), "\n");

    sd.values = a * age + b
    
    write.table(list(age=age, mean=mean.values, sd=sd.values), paste("../tables/", name, ".txt", sep=""), sep="\t", na="", row.names=FALSE, col.names=FALSE)
}

hand_length <- function(mean.span, sd.span, age=def.age) {
    mean.data <- read.table("../data/combined/HL.txt")
    sd.data <- read.table("../data/madhulika/HL.txt")

    mean.model <- loess(V2 ~ V1, mean.data, weights=1/V3, span=mean.span, surface="direct", statistics="exact", trace.hat="exact")
    mean.values <- predict(mean.model, age)

    sd.model <- loess(V3 ~ V1, sd.data, weights=1/V3, span=sd.span, surface="direct", statistics="exact", trace.hat="exact")
    sd.values <- predict(sd.model, age)
    sd.values[13:31 - 13 + 1] <- NA  # Madhulika's data isn't trustworthy at
    				     #                               young ages

    write.table(list(age=age, mean=mean.values, sd=sd.values), paste("../tables/HL.txt", sep=""), sep="\t", na="", row.names=FALSE, col.names=FALSE)
}

normal_loess("Body",  0.3, 0.6)
normal_loess("Brain", 0.3, 0.6)
split_loess("Thymus", 0.2, 0.3, 0.4, 0.6)
normal_loess("Lungs", 0.4, 0.6)
split_loess("Heart", 0.2, 0.6, 0.4, 0.6)
normal_loess("Liver", 0.3, 0.5)
split_loess("Spleen", 0.2, 0.4, 0.3, 0.4)
split_loess("Adrenals", 0.2, 0.3, 0.4, 0.7)
normal_loess("Pancreas", 0.4, 0.8, 20:42)
normal_loess("Kidneys", 0.3, 0.6)

normal_loess("CHL", 0.4, 0.6, 12:42)
normal_loess("CRL", 0.4, 0.7, 12:42)
quadratic_linear("HC", "guihard", 12:42)
quadratic_linear("BPD", "guihard", 12:42)
quadratic_linear("OCD", "guihard", 12:42)
quadratic_linear("ICD", "guihard", 12:42)
linear("PL", "chambers", 13:26)
linear("CC", "chambers", 13:40)
linear_linear("IND", "guihard", 12:42)
linear("AC", "chambers", 13:26)
hand_length(0.6, 0.9, 13:42)
normal_loess("FL", 0.4, 0.8, 12:42)
